@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/log.c,v 1.11 2007/01/08 10:50:18 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2007 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing log files. The code for maintaining datestamped
@@ -35,7 +37,6 @@ static uschar *rejectlog_datestamp = NULL;
 
 static int    mainlogfd = -1;
 static int    rejectlogfd = -1;
-static int    processlogfd = -1;
 static ino_t  mainlog_inode = 0;
 static ino_t  rejectlog_inode = 0;
 
@@ -56,7 +57,9 @@ static uschar *file_path = US"";
 
 /* The given string is split into sections according to length, or at embedded
 newlines, and syslogged as a numbered sequence if it is overlong or if there is
-more than one line.
+more than one line. However, if we are running in the test harness, do not do
+anything. (The test harness doesn't use syslog - for obvious reasons - but we
+can get here if there is a failure to open the panic log.)
 
 Arguments:
   priority       syslog priority
@@ -71,6 +74,8 @@ write_syslog(int priority, uschar *s)
 int len, pass;
 int linecount = 0;
 
+if (running_in_test_harness) return;
+
 if (!syslog_timestamp) s += log_timezone? 26 : 20;
 
 len = Ustrlen(s);
@@ -313,15 +318,16 @@ open, arrange for automatic closure on exec(), and then return. */
 
 if (*fd >= 0)
   {
-  fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
+  (void)fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
   return;
   }
 
 /* Open was not successful: try creating the file. If this is a root process,
 we must do the creating in a subprocess set to exim:exim in order to ensure
 that the file is created with the right ownership. Otherwise, there can be a
-race if an exim process is trying to write to the log at the same time. The use
-of SIGUSR1 by the exiwhat utility can provoke a lot of simultaneous writing. */
+race if another Exim process is trying to write to the log at the same time.
+The use of SIGUSR1 by the exiwhat utility can provoke a lot of simultaneous
+writing. */
 
 euid = geteuid();
 
@@ -349,17 +355,23 @@ else if (euid == root_uid)
     _exit((create_log(buffer) < 0)? 1 : 0);
     }
 
-  /* Wait for the subprocess. If it succeeded retry the open. */
+  /* If we created a subprocess, wait for it. If it succeeded retry the open. */
 
-  while (waitpid(pid, &status, 0) != pid);
-  if (status == 0) *fd = Uopen(buffer, O_APPEND|O_WRONLY, LOG_MODE);
+  if (pid > 0)
+    {
+    while (waitpid(pid, &status, 0) != pid);
+    if (status == 0) *fd = Uopen(buffer, O_APPEND|O_WRONLY, LOG_MODE);
+    }
+
+  /* If we failed to create a subprocess, we are in a bad way. We fall through
+  with *fd still < 0, and errno set, letting the code below handle the error. */
   }
 
 /* If we now have an open file, set the close-on-exec flag and return. */
 
 if (*fd >= 0)
   {
-  fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
+  (void)fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
   return;
   }
 
@@ -431,6 +443,43 @@ return ptr + 4;
 }
 
 
+/*************************************************
+*           A write() operation failed           *
+*************************************************/
+
+/* This function is called when write() fails on anything other than the panic
+log, which can happen if a disk gets full or a file gets too large or whatever.
+We try to save the relevant message in the panic_save buffer before crashing
+out.
+
+Arguments:
+  name      the name of the log being written
+  length    the string length being written
+  rc        the return value from write()
+
+Returns:    does not return
+*/
+
+static void
+log_write_failed(uschar *name, int length, int rc)
+{
+int save_errno = errno;
+
+if (panic_save_buffer == NULL)
+  {
+  panic_save_buffer = (uschar *)malloc(LOG_BUFFER_SIZE);
+  if (panic_save_buffer != NULL)
+    memcpy(panic_save_buffer, log_buffer, LOG_BUFFER_SIZE);
+  }
+
+log_write(0, LOG_PANIC_DIE, "failed to write to %s: length=%d result=%d "
+  "errno=%d (%s)", name, length, rc, save_errno,
+  (save_errno == 0)? "write incomplete" : strerror(save_errno));
+/* Never returns */
+}
+
+
+
 /*************************************************
 *            Write message to log file           *
 *************************************************/
@@ -474,6 +523,8 @@ Malloc is used directly because the store functions may call log_write().
 If a message_id exists, we include it after the timestamp.
 
 Arguments:
+  selector  write to main log or LOG_INFO only if this value is zero, or if
+              its bit is set in log_write_selector
   flags     each bit indicates some independent action:
               LOG_SENDER      add raw sender to the message
               LOG_RECIPIENTS  add raw recipients list to message
@@ -485,8 +536,6 @@ Arguments:
               LOG_PANIC       write to panic log or syslog LOG_ALERT
               LOG_PANIC_DIE   write to panic log or LOG_ALERT and then crash
               LOG_PROCESS     write to process log (always a file)
-  selector  write to main log or LOG_INFO only if this value is zero, or if
-              its bit is set in log_selector
   format    a printf() format
   ...       arguments for format
 
@@ -497,6 +546,7 @@ void
 log_write(unsigned int selector, int flags, char *format, ...)
 {
 uschar *ptr;
+int length, rc;
 int paniclogfd;
 va_list ap;
 
@@ -624,16 +674,22 @@ DEBUG(D_any|D_v)
   Ustrcpy(ptr, "LOG:");
   ptr += 4;
 
+  /* Show the options that were passed into the call. These are those whose
+  flag values do not have the 0x80000000 bit in them. Note that this
+  automatically exclude the "all" setting. */
+
   for (i = 0; i < log_options_count; i++)
     {
-    if (log_options[i].bit == L_all) continue;
-    if ((selector & log_options[i].bit) != 0)
+    unsigned int bit = log_options[i].bit;
+    if ((bit & 0x80000000) != 0) continue;
+    if ((selector & bit) != 0)
       {
       *ptr++ = ' ';
       Ustrcpy(ptr, log_options[i].name);
       while (*ptr) ptr++;
       }
     }
+
   sprintf(CS ptr, "%s%s%s%s%s\n  ",
     ((flags & LOG_MAIN) != 0)?    " MAIN"   : "",
     ((flags & LOG_PANIC) != 0)?   " PANIC"  : "",
@@ -668,6 +724,10 @@ if (disable_logging)
   return;
   }
 
+/* Handle disabled reject log */
+
+if (!write_rejectlog) flags &= ~LOG_REJECT;
+
 /* Create the main message in the log buffer, including the message
 id except for the process log and when called by a utility. */
 
@@ -689,7 +749,7 @@ va_end(ap);
 this way because it kind of fits with LOG_RECIPIENTS. */
 
 if ((flags & LOG_SENDER) != 0 &&
-    ptr < log_buffer + LOG_BUFFER_SIZE - 8 - Ustrlen(raw_sender))
+    ptr < log_buffer + LOG_BUFFER_SIZE - 10 - Ustrlen(raw_sender))
   {
   sprintf(CS ptr, " from <%s>", raw_sender);
   while (*ptr) ptr++;
@@ -716,6 +776,7 @@ if ((flags & LOG_RECIPIENTS) != 0 && ptr < log_buffer + LOG_BUFFER_SIZE - 6 &&
 
 sprintf(CS  ptr, "\n");
 while(*ptr) ptr++;
+length = ptr - log_buffer;
 
 /* Handle loggable errors when running a utility, or when address testing.
 Write to log_stderr unless debugging (when it will already have been written),
@@ -724,7 +785,7 @@ or unless there is no log_stderr (expn called from daemon, for example). */
 if (!really_exim || log_testing_mode)
   {
   if (debug_selector == 0 && log_stderr != NULL &&
-      (selector == 0 || (selector & log_selector) != 0))
+      (selector == 0 || (selector & log_write_selector) != 0))
     {
     if (host_checking)
       fprintf(log_stderr, "LOG: %s", CS(log_buffer + 20));  /* no timestamp */
@@ -742,7 +803,7 @@ has been renamed. Therefore, do a stat() and see if the inode has changed, and
 if so, re-open. */
 
 if ((flags & LOG_MAIN) != 0 &&
-    (selector == 0 || (selector & log_selector) != 0))
+    (selector == 0 || (selector & log_write_selector) != 0))
   {
   if ((logging_mode & LOG_MODE_SYSLOG) != 0 &&
       (syslog_duplication || (flags & (LOG_REJECT|LOG_PANIC)) == 0))
@@ -761,7 +822,7 @@ if ((flags & LOG_MAIN) != 0 &&
       uschar *nowstamp = tod_stamp(tod_log_datestamp);
       if (Ustrncmp (mainlog_datestamp, nowstamp, Ustrlen(nowstamp)) != 0)
         {
-        close(mainlogfd);             /* Close the file */
+        (void)close(mainlogfd);       /* Close the file */
         mainlogfd = -1;               /* Clear the file descriptor */
         mainlog_inode = 0;            /* Unset the inode */
         mainlog_datestamp = NULL;     /* Clear the datestamp */
@@ -777,7 +838,7 @@ if ((flags & LOG_MAIN) != 0 &&
       {
       if (Ustat(mainlog_name, &statbuf) < 0 || statbuf.st_ino != mainlog_inode)
         {
-        close(mainlogfd);
+        (void)close(mainlogfd);
         mainlogfd = -1;
         mainlog_inode = 0;
         }
@@ -791,11 +852,18 @@ if ((flags & LOG_MAIN) != 0 &&
       if (fstat(mainlogfd, &statbuf) >= 0) mainlog_inode = statbuf.st_ino;
       }
 
-    write(mainlogfd, log_buffer, ptr - log_buffer);
+    /* Failing to write to the log is disastrous */
+
+    if ((rc = write(mainlogfd, log_buffer, length)) != length)
+      {
+      log_write_failed(US"main log", length, rc);
+      /* That function does not return */
+      }
     }
   }
 
-/* Handle the log for rejected messages: if there are any header lines (i.e. if
+/* Handle the log for rejected messages. This can be globally disabled, in
+which case the flags are altered above. If there are any header lines (i.e. if
 the rejection is happening after the DATA phase), log the recipients and the
 headers. */
 
@@ -803,7 +871,7 @@ if ((flags & LOG_REJECT) != 0)
   {
   header_line *h;
 
-  if (header_list != NULL && (log_selector & L_rejected_header) != 0)
+  if (header_list != NULL && (log_extra_selector & LX_rejected_header) != 0)
     {
     if (recipients_count > 0)
       {
@@ -836,9 +904,13 @@ if ((flags & LOG_REJECT) != 0)
         }
       }
 
+    /* A header with a NULL text is an unfilled in Received: header */
+
     for (h = header_list; h != NULL; h = h->next)
       {
-      BOOL fitted = string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+      BOOL fitted;
+      if (h->text == NULL) continue;
+      fitted = string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
         "%c %s", h->type, h->text);
       while(*ptr) ptr++;
       if (!fitted)         /* Buffer is full; truncate */
@@ -850,6 +922,8 @@ if ((flags & LOG_REJECT) != 0)
         break;
         }
       }
+
+    length = ptr - log_buffer;
     }
 
   /* Write to syslog or to a log file */
@@ -864,14 +938,14 @@ if ((flags & LOG_REJECT) != 0)
 
   if ((logging_mode & LOG_MODE_FILE) != 0)
     {
-     struct stat statbuf;
+    struct stat statbuf;
 
     if (rejectlog_datestamp != NULL)
       {
       uschar *nowstamp = tod_stamp(tod_log_datestamp);
       if (Ustrncmp (rejectlog_datestamp, nowstamp, Ustrlen(nowstamp)) != 0)
         {
-        close(rejectlogfd);             /* Close the file */
+        (void)close(rejectlogfd);       /* Close the file */
         rejectlogfd = -1;               /* Clear the file descriptor */
         rejectlog_inode = 0;            /* Unset the inode */
         rejectlog_datestamp = NULL;     /* Clear the datestamp */
@@ -888,7 +962,7 @@ if ((flags & LOG_REJECT) != 0)
       if (Ustat(rejectlog_name, &statbuf) < 0 ||
            statbuf.st_ino != rejectlog_inode)
         {
-        close(rejectlogfd);
+        (void)close(rejectlogfd);
         rejectlogfd = -1;
         rejectlog_inode = 0;
         }
@@ -902,7 +976,11 @@ if ((flags & LOG_REJECT) != 0)
       if (fstat(rejectlogfd, &statbuf) >= 0) rejectlog_inode = statbuf.st_ino;
       }
 
-    write(rejectlogfd, log_buffer, ptr - log_buffer);
+    if ((rc = write(rejectlogfd, log_buffer, length)) != length)
+      {
+      log_write_failed(US"reject log", length, rc);
+      /* That function does not return */
+      }
     }
   }
 
@@ -914,39 +992,55 @@ written to a file - never to syslog. */
 
 if ((flags & LOG_PROCESS) != 0)
   {
-  if (processlogfd < 0)
-    open_log(&processlogfd, lt_process);  /* No return on error */
-  write(processlogfd, log_buffer, ptr - log_buffer);
+  int processlogfd;
+  open_log(&processlogfd, lt_process);  /* No return on error */
+  if ((rc = write(processlogfd, log_buffer, length)) != length)
+    {
+    log_write_failed(US"process log", length, rc);
+    /* That function does not return */
+    }
+  (void)close(processlogfd);
   }
 
 
 /* Handle the panic log, which is not kept open like the others. If it fails to
 open, there will be a recursive call to log_write(). We detect this above and
 attempt to write to the system log as a last-ditch try at telling somebody. In
-all cases, try to write to log_stderr. */
+all cases except mua_wrapper, try to write to log_stderr. */
 
 if ((flags & LOG_PANIC) != 0)
   {
-  if (log_stderr != NULL && log_stderr != debug_file)
+  if (log_stderr != NULL && log_stderr != debug_file && !mua_wrapper)
     fprintf(log_stderr, "%s", CS log_buffer);
 
-  /* If this panic logging was caused by a failure to open the main log,
-  the original log line is in panic_save_buffer. */
-
   if ((logging_mode & LOG_MODE_SYSLOG) != 0)
     {
     write_syslog(LOG_ALERT, log_buffer);
     }
 
+  /* If this panic logging was caused by a failure to open the main log,
+  the original log line is in panic_save_buffer. Make an attempt to write it. */
+
   if ((logging_mode & LOG_MODE_FILE) != 0)
     {
     panic_recurseflag = TRUE;
     open_log(&paniclogfd, lt_panic);  /* Won't return on failure */
     panic_recurseflag = FALSE;
+
     if (panic_save_buffer != NULL)
-      write(paniclogfd, panic_save_buffer, Ustrlen(panic_save_buffer));
-    write(paniclogfd, log_buffer, ptr - log_buffer);
-    close(paniclogfd);
+      (void) write(paniclogfd, panic_save_buffer, Ustrlen(panic_save_buffer));
+
+    if ((rc = write(paniclogfd, log_buffer, length)) != length)
+      {
+      int save_errno = errno;
+      write_syslog(LOG_CRIT, log_buffer);
+      sprintf(CS log_buffer, "write failed on panic log: length=%d result=%d "
+        "errno=%d (%s)", length, rc, save_errno, strerror(save_errno));
+      write_syslog(LOG_CRIT, log_buffer);
+      flags |= LOG_PANIC_DIE;
+      }
+
+    (void)close(paniclogfd);
     }
 
   /* Give up if the DIE flag is set */
@@ -966,11 +1060,9 @@ void
 log_close_all(void)
 {
 if (mainlogfd >= 0)
-  { close(mainlogfd); mainlogfd = -1; }
-if (processlogfd >= 0)
-  { close(processlogfd); processlogfd = -1; }
+  { (void)close(mainlogfd); mainlogfd = -1; }
 if (rejectlogfd >= 0)
-  { close(rejectlogfd); rejectlogfd = -1; }
+  { (void)close(rejectlogfd); rejectlogfd = -1; }
 closelog();
 syslog_open = FALSE;
 }
